import {
  ProseEditor,
  Toolbar
} from 'substance'

import ContainerEditor from '../ContainerEditor'
import Comments from '../panes/Comments/CommentBoxList'
import CommentsProvider from '../panes/Comments/CommentsProvider'
import TrackChangesProvider from '../elements/track_change/TrackChangesProvider'
class MiniEditor extends ProseEditor {

  didMount () {
    this.context.notesProvider.config.miniEditorContext = this.getChildContext()
    this.context.notesProvider.config.miniEditorSession = this.editorSession
    this.on('noteSelected', this.scrollTo, this)
  }

  render ($$) {
    const el = $$('div').addClass('sc-mini-editor')
    let toolbar = this._renderMiniToolbar($$)
    let editor = this._renderEditor($$)
    let SplitPane = this.componentRegistry.get('split-pane')
    let ScrollPane = this.componentRegistry.get('scroll-pane')
    let Overlay = this.componentRegistry.get('overlay')

    var commentsPane = $$(Comments, {
      comments: this.props.comments || {},
      fragment: this.props.fragment,
      update: this.props.update,
      user: this.props.user
    }).addClass('sc-comments-pane')

    const editorWithComments = $$(SplitPane, { sizeA: '100%', splitType: 'vertical' })
      .append(
        editor,
        commentsPane
      )

    const contentPanel = $$(ScrollPane, {
      name: 'miniEditorContentPanel',
      scrollbarPosition: 'right'
    })
      .append(editorWithComments, $$(Overlay))
      .attr('id', 'mini-editor-content-panel')
      .ref('miniEditorContentPanel')

    el.append(
      $$(SplitPane, { splitType: 'horizontal' })
        .append(toolbar, contentPanel)
    )

    return el
  }

  _renderMiniToolbar ($$) {
    let commandStates = this.commandManager.getCommandStates()
    return $$('div').addClass('se-toolbar-wrapper').append(
      $$(Toolbar, {
        commandStates: commandStates,
        toolGroups: ['document', 'annotations']
      }).ref('mini_toolbar')
    )
  }

  _renderEditor ($$) {
    return $$(ContainerEditor, {
      comments: this.props.comments,
      containerId: this.props.containerId,
      editorSession: this.editorSession,
      disabled: this.props.disabled,
      fragment: this.props.fragment,
      spellcheck: 'native',
      trackChanges: this.props.trackChanges,
      user: this.props.user
    }).ref('mini_body')
  }

  scrollTo (nodeId) {
    this.refs.miniEditorContentPanel.scrollTo(nodeId)
  }

  getChildContext () {
    const oldContext = super.getChildContext()
    const doc = this.doc

    // comments provider
    const commentsProvider = new CommentsProvider(doc, {
      commandManager: this.commandManager,
      comments: this.props.fragment.comments,
      containerId: this.props.containerId,
      controller: this,
      editorSession: this.editorSession,
      fragment: this.props.fragment,
      surfaceManager: this.surfaceManager,
      toggleCommentsArea: this.toggleCommentsArea,
      update: this.props.update
    })

    const trackChangesProvider = new TrackChangesProvider(doc, {
      commandManager: this.commandManager,
      containerId: this.props.containerId,
      controller: this,
      editorSession: this.editorSession,
      surfaceManager: this.surfaceManager,
      user: this.props.user
    })

    // attach all to context
    return {
      ...oldContext,
      commentsProvider,
      trackChangesProvider
    }
  }
}

export default MiniEditor
